% enc/dec script for comparison of Full Search and Diamond Search
% For luma components, the use of FS/DS must be hardcoded in obmc.m, by
% commenting out one of line 15 or 16. Run the Full search cell and Diamond
% search cell one at a time.

clear all

% ******** Encoder Parameters ***********
origFileName = 'C:\zzz_m\test_environment\original_sequences\CrowdRun_720p25.yuv';    % Name of YUV4:2:0 file to encode
firstFrame = 0;                 % First frame to encode (0-indexed)
lastFrame = 24;                  % Last frame to encode (0-indexed)
frameWidth = 1280;                    % Frame width
frameHeight = 720;                   % Frame height
frameRate = 25;               % Frame rate
lumaBlockSize = 40;            % Block size for Motion Estimation (Luma, halved for chroma components)
lumaSearchRange = 64;          % Search range for Motion Estimation (Luma, halved for chroma components)
filterType = '1/3';            % Filter type for MCTF (Haar or 1/3 filter)
numOfTemporalLevels = 3;        % Number of temporal levels for MCTF
                                % decomposition
mvEntropyCoder = 1;             % 0: Canonical Huffman 1: Arithmetic coding
targetRate = 10000;                 % Target output bitrate [kbps] (0 = lossless JP2k coding)

lumaBudgetPercent = 0.9;
CbBudgetPercent = 0.05;
lpPercent = 0.25;
% *******************************


%%
% Full search =============================================================
searchType = 'full';         % Motion Estimation search type
encOutputFolder = 'C:\zzz_m\BMA_comparison\FS\str\CrowdRun_10000kbps_FS';        % Name of output folder
logfile = 'C:\zzz_m\BMA_comparison\FS\log_CrowdRun_10000kbps_FS.txt';

enc = Encoder(origFileName, firstFrame, lastFrame, frameWidth, frameHeight,...
               frameRate, lumaBlockSize, lumaSearchRange, searchType, filterType,...
               numOfTemporalLevels, targetRate, mvEntropyCoder, encOutputFolder, lumaBudgetPercent, CbBudgetPercent, lpPercent, logfile );
           
encode(enc);
clear enc

% Decoder parameters
decOutputFolder = 'C:\zzz_m\BMA_comparison\FS\rec\CrowdRun_10000kbps_FS';               % Folder to put reconstructed files
outputFileName = 'rec_CrowdRun_10000_FS.yuv';         % Output yuv file

d = Decoder(encOutputFolder, decOutputFolder, outputFileName, logfile, origFileName);

decode(d);
clear d
%==========================================================================
%%






% Diamond search ==========================================================
searchType = 'diamond';         % Motion Estimation search type
encOutputFolder = 'C:\zzz_m\BMA_comparison\DS\str\CrowdRun_10000kbps_DS';        % Name of output folder
logfile = 'C:\zzz_m\BMA_comparison\DS\log_CrowdRun_10000kbps_DS.txt';

enc = Encoder(origFileName, firstFrame, lastFrame, frameWidth, frameHeight,...
               frameRate, lumaBlockSize, lumaSearchRange, searchType, filterType,...
               numOfTemporalLevels, targetRate, mvEntropyCoder, encOutputFolder, lumaBudgetPercent, CbBudgetPercent, lpPercent, logfile );
           
encode(enc);
clear enc

% Decoder parameters
decOutputFolder = 'C:\zzz_m\BMA_comparison\DS\rec\CrowdRun_10000kbps_DS';               % Folder to put reconstructed files
outputFileName = 'rec_CrowdRun_10000_DS.yuv';         % Output yuv file

d = Decoder(encOutputFolder, decOutputFolder, outputFileName, logfile, origFileName);

decode(d);
clear d
%==========================================================================
