classdef FiveThreeMCTF
    %TEST
    
    methods (Static)
        
        function [subband_frames, mvx, mvy] = analysis(frames, idr, numOfTemporalLevels, blockSize, searchRange, searchType)
            [frame_height, frame_width, GOP_size] = size(frames);
            
            mvx = zeros(frame_height / blockSize, frame_width / blockSize, 2^(numOfTemporalLevels+1) - 2);
            mvy = zeros(frame_height / blockSize, frame_width / blockSize, 2^(numOfTemporalLevels+1) - 2);
            subband_frames = zeros(frame_height, frame_width, GOP_size);
            
            lowpass_frames = zeros(frame_height, frame_width, GOP_size + 1);
            lowpass_frames(:,:,1) = idr;
            for i = 1:GOP_size
                lowpass_frames(:,:,i+1) = frames(:,:,i);
            end
            
            level_index = numOfTemporalLevels;
            while( level_index > 0)
                h_frames_temp = zeros(frame_height, frame_width, 2^(numOfTemporalLevels-1));
                for i = 1:2:2^level_index
                    frame_0 = lowpass_frames(:, :, i);
                    frame_1 = lowpass_frames(:, :, i+1);
                    frame_2 = lowpass_frames(:, :, i+2);
                    
                    [mvx_f_temp, mvy_f_temp] = block_matching_motion_estimation(frame_1, frame_0, blockSize, searchRange, searchType);
                    [mvx_b_temp, mvy_b_temp] = block_matching_motion_estimation(frame_1, frame_2, blockSize, searchRange, searchType);
                    h_temp = double(frame_1) - FiveThreeMCTF.prediction(frame_0, frame_2, mvx_f_temp, mvy_f_temp, mvx_b_temp, mvy_b_temp, blockSize);
                    
                    mvx(:, :, 2^(level_index)-1+(i-1)) = mvx_f_temp;
                    mvx(:, :, 2^(level_index)+(i-1)) = mvx_b_temp;
                    
                    mvy(:, :, 2^(level_index)-1+(i-1)) = mvy_f_temp;
                    mvy(:, :, 2^(level_index)+(i-1)) = mvy_b_temp;
                    
                    h_frames_temp(:, :, ceil(i/2)) = h_temp;
                end
                
                c = 1;
                for i = 2^(level_index-1)+1 : 2^(level_index)
                    subband_frames(:, :, i) = h_frames_temp(:,:,c);
                end
                
                lowpass_frames = lowpass_frames(:,:,1:2:2^level_index+1);
                mv_cntr = 2^numOfTemporalLevels;
                for i = 2:2^(level_index-1)
                    lowpass_frames(:,:,i) = double( lowpass_frames(:,:,i) ) + FiveThreeMCTF.update(h_frames_temp(:, :, i-1), ...
                        h_frames_temp(:, :, i), mvx(:,:,mv_cntr), mvy(:,:,mv_cntr), ...
                        mvx(:,:,mv_cntr+1), mvy(:,:,mv_cntr+1), blockSize );
                    mv_cntr = mv_cntr + 2;
                end
                level_index = level_index - 1;
            end
            subband_frames(:, :, 1) = frames(:, :, GOP_size);            
            
        end
        
        function reconstructed_frames = synthesis(subband_frames, idr, mvx, mvy, blockSize, numOfTemporalLevels)            
            [frame_height, frame_width, GOP_size] = size(subband_frames);
            
            lowpass_frames = zeros(frame_height, frame_width, 2);%GOP_size+1);
            lowpass_frames(:,:,1) = idr; % IDR            
            lowpass_frames(:,:,3) = subband_frames(:, :, 1);
            lowpass_frames(:,:,2) = double( subband_frames(:,:,2) ) + FiveThreeMCTF.prediction(lowpass_frames(:,:,1), lowpass_frames(:,:,3), ...
                mvx(:,:,1), mvy(:,:,1), mvx(:,:,2), mvy(:,:,2), blockSize);
            
            level_index = 1;
            mv_cntr_prd = 3;
            h_cntr = 3;
            while( level_index < numOfTemporalLevels)
                lowpass_frames_temp = zeros(frame_height, frame_width, 2^(level_index+1)+1);  
                mv_cntr_upd = 2^(level_index+1);
                
                %Update pass
                for i = 2:2^level_index %inner lp bands -> indx 2,3,..., (2^level_index+1)-1 indx 1 = idr
                    h_frame_0 = subband_frames(:, :, h_cntr);
                    h_frame_1 = subband_frames(:, :, h_cntr + 1);
                    h_cntr = h_cntr + 1;
                    
                    lowpass_frames(:,:,i) = double( lowpass_frames(:,:,i) ) - FiveThreeMCTF.update(h_frame_0, h_frame_1, mvx(:, :, mv_cntr_upd), mvy(:, :, mv_cntr_upd), mvx(:, :, mv_cntr_upd+1), mvy(:, :, mv_cntr_upd+1), blockSize);
                    mv_cntr_upd = mv_cntr_upd + 2;                                                            
                end
                %Prediction pass
                c = 1;
                for i = 1:2:2^(level_index+1)+1 %fill temp array
                    lowpass_frames_temp(:,:,i) = lowpass_frames(:,:,c);
                    c = c + 1;
                end
                h_cnt2 = 2^level_index+1;
                for i = 2:2:2^(level_index+1)+1
                    lowpass_frames_temp(:,:,i) = double( subband_frames(:,:,h_cnt2) ) + FiveThreeMCTF.prediction( lowpass_frames_temp(:,:,i-1), lowpass_frames_temp(:,:,i+1), mvx(:,:,mv_cntr_prd), mvy(:,:,mv_cntr_prd), mvx(:,:,mv_cntr_prd+1), mvy(:,:,mv_cntr_prd+1), blockSize );
                    mv_cntr_prd = mv_cntr_prd + 2;
                    h_cnt2 = h_cnt2 + 1;
                end
                h_cntr = h_cntr + 1;
                level_index = level_index + 1;
                lowpass_frames = lowpass_frames_temp;
            end
            reconstructed_frames = uint8(lowpass_frames(:,:,2:GOP_size+1)); %Convert from double to uint8           
            
        end
        
    end
    
    methods (Static, Access = private)
        
        function a = prediction(prev_frame, fut_frame, mvx_f, mvy_f, mvx_b, mvy_b, blockSize)
            dim = size(prev_frame);
            a = zeros(dim);
            for i = 1:dim(1)
                for j = 1:dim(2)
                    a(i,j) = floor( ...
                        (prev_frame( i + mvy_f( ceil(i/blockSize), ceil(j/blockSize) ), j + mvx_f( ceil(i/blockSize), ceil(j/blockSize) ) ) +...
                        fut_frame( i + mvy_b( ceil(i/blockSize), ceil(j/blockSize) ), j + mvx_b( ceil(i/blockSize), ceil(j/blockSize) ) ) ...
                        )/ 2 );
                end
            end
        end
        
        function a = update(h_0, h_1, mvx_f, mvy_f, mvx_b, mvy_b, blockSize)
            dim = size(h_0);
            a = zeros(dim);
            for i = 1:dim(1)
                for j = 1:dim(2)
                    a( i + mvy_f( ceil(i/blockSize),ceil(j/blockSize) ), j + mvx_f( ceil(i/blockSize),ceil(j/blockSize) ) ) =  h_0(i, j) / 4;
                end
            end
            for i = 1:dim(1)
                for j = 1:dim(2)
                    a( i + mvy_b( ceil(i/blockSize),ceil(j/blockSize) ), j + mvx_b( ceil(i/blockSize),ceil(j/blockSize) ) ) = ...
                        a( i + mvy_b( ceil(i/blockSize),ceil(j/blockSize) ), j + mvx_b( ceil(i/blockSize),ceil(j/blockSize) ) ) + ...
                        h_1(i, j) / 4  + (1/2); %er + 1/2 nodvendig? uanz ikke aarsak til ghosting                    
                end
            end
            for i = 1:dim(1)
                for j = 1:dim(2)
                    a(i,j) = floor( a(i,j) );
                end
            end
        end
        
    end
    
end