classdef HaarMCTF
    
    methods (Static)
        
        function [l_band, h_band] = analysis(A, B, mvx_f, mvy_f, blockSize)
            if( size(A) ~= size(B) )
                error('A and B not of equal size');
            end
            
            h_band = double(B) - HaarMCTF.prediction(A, mvx_f, mvy_f, blockSize);
            l_band = double(A) + HaarMCTF.update(h_band, mvx_f, mvy_f, blockSize);
        end
        
        function [A, B] = synthesis(l_band, h_band, mvx_f, mvy_f, blockSize)
            if( size(l_band) ~= size(h_band) )
                error('l_band and h_band not of equal size');
            end
            
            A = double(l_band) - HaarMCTF.update(h_band, mvx_f, mvy_f, blockSize);
            B = double(h_band) + HaarMCTF.prediction(A, mvx_f, mvy_f, blockSize);            
        end
        
    end
    
    methods (Static, Access = private)
       
        function a = prediction(frame, mvx_f, mvy_f, blockSize)
            dim = size(frame);
            a = zeros(dim);
            for i = 1:dim(1)
                for j = 1:dim(2)
                    a(i,j) = floor( frame( i + mvy_f( ceil(i/blockSize), ceil(j/blockSize) ), j + mvx_f( ceil(i/blockSize), ceil(j/blockSize) ) ) );
                end
            end
        end
        
        function a = update(frame, mvx_f, mvy_f, blockSize)
            dim = size(frame);
            a = zeros(dim);
            for i = 1:dim(1)
                for j = 1:dim(2)
                    a( i + mvy_f( ceil(i/blockSize),ceil(j/blockSize) ), j + mvx_f( ceil(i/blockSize),ceil(j/blockSize) ) ) = floor( frame(i, j) / 2 );
                end
            end
        end
        
    end
    
end