classdef OneThreeMCTF
    
    methods (Static)
        
        function h_band = analysis(A_1, B, A_2, mvx_f, mvy_f, mvx_b, mvy_b, blockSize, varargin)
            obmc = 1;
            if (nargin == 9)
                obmc = varargin{1};
            end
            
            h_band = double(B) - OneThreeMCTF.prediction(A_1, A_2, mvx_f, mvy_f, mvx_b, mvy_b, blockSize, obmc);
            
        end
        
        function L = synthesis(l_band_1, h_band, l_band_2, mvx_f, mvy_f, mvx_b, mvy_b, blockSize, varargin)
            obmc = 1;
            if (nargin == 9)
                obmc = varargin{1};
            end
            
            L = double(h_band) + OneThreeMCTF.prediction(l_band_1, l_band_2, mvx_f, mvy_f, mvx_b, mvy_b, blockSize, obmc);
            
        end
        
    end
    
    methods (Static, Access = private)
       
        function a = prediction(prev_frame, fut_frame, mvx_f, mvy_f, mvx_b, mvy_b, blockSize, obmc)
            dim = size(prev_frame);
            a = zeros(dim);
            
            if (obmc)
                b = blockSize / 2;
                
                weights = ones( dim / b ) * 2;
                weights(1,1) = 1;
                weights(1,end) = 1;
                weights(end,1) = 1;
                weights(end,end) = 1;
                weights(2:end-1,2:end-1) = 4;
                
                mvDim = size(mvx_f);
                for i = 1:mvDim(1)
                    for j = 1:mvDim(2)
                        dx = mvx_f(i,j);
                        dy = mvy_f(i,j);
                        for y = (i-1)*b+1 : (i-1)*b+blockSize
                            for x = (j-1)*b+1 : (j-1)*b+blockSize
                                a(y,x) = a(y,x) + ( prev_frame(y+dy, x+dx) / weights(ceil(y/b),ceil(x/b)) );
                            end
                        end
                    end
                end
                
                mvDim = size(mvx_b);
                bb = zeros(dim);
                for i = 1:mvDim(1)
                    for j = 1:mvDim(2)
                        dx = mvx_b(i,j);
                        dy = mvy_b(i,j);
                        for y = (i-1)*b+1 : (i-1)*b+blockSize
                            for x = (j-1)*b+1 : (j-1)*b+blockSize
                                bb(y,x) = bb(y,x) + (fut_frame(y+dy, x+dx) / weights(ceil(y/b),ceil(x/b)));
                            end
                        end
                    end
                end
                
                a = floor( (a + bb) /2 );
            
            else %not obmc            
                for i = 1:dim(1)
                    for j = 1:dim(2)
                        a(i,j) = floor( ...
                            (prev_frame( i + mvy_f( ceil(i/blockSize), ceil(j/blockSize) ), j + mvx_f( ceil(i/blockSize), ceil(j/blockSize) ) ) +...
                            fut_frame( i + mvy_b( ceil(i/blockSize), ceil(j/blockSize) ), j + mvx_b( ceil(i/blockSize), ceil(j/blockSize) ) ) ...
                            )/ 2 );
                    end
                end
            end
        end                
        
    end
    
end