function [mvx, mvy] = block_matching_motion_estimation(cur_frame, ref_frame, blockSize, searchRange, searchType)
% Search range: searching in area [-(R-1), (R-1)]

if ( rem(searchRange,1) ~= 0 )
    error('searchRange parameter must be integer valued');
end
if( strcmpi(searchType, 'diamond') )
    [mvx, mvy] = diamondSearch(cur_frame, ref_frame, blockSize, searchRange);
elseif( strcmpi(searchType, 'full') )
    [mvx, mvy] = fullSearch(cur_frame, ref_frame, blockSize, searchRange);
else
    error('Parameter searchType must be eiter "full" or "diamond"');
end

end



function [mvx, mvy] = diamondSearch(cur_frame, ref_frame, blockSize, searchRange)
% Diamond Search

ldsp_ind = [0 2; -1 1; 1 1; -2 0; 0 0; 2 0; -1 -1; 1 -1; 0 -2]; %Large Diamond Search Pattern indices
sdsp_ind = [0 1; -1 0; 0 0; 1 0; 0 -1]; %Small Diamond Search Pattern indices

dim = size(cur_frame);
mvx = zeros(dim / blockSize);
mvy = zeros(dim / blockSize);

for i = 1:blockSize:dim(1)
    for j = 1:blockSize:dim(2)
        MSE_min = 256^2;
        dy = 0;     %Current y vector comp
        dx = 0;     %Current x vector comp
        %ldsp
        while ( abs(dy) < searchRange - 3 && abs(dx) < searchRange - 3) %Less than search range - 3 because ldsp might add 2, and sdsp might add 1
            best_y = 0; %Best y relative ind
            best_x = 0; %Best x relative ind
            for k = 1:length(ldsp_ind)
                ref_y = i + dy + ldsp_ind(k,2);
                ref_x = j + dx + ldsp_ind(k,1);
                if ( ref_y > 0 && (ref_y + blockSize - 1) <= dim(1) ...
                        && ref_x > 0 && (ref_x + blockSize - 1) <= dim(2) ) %Check if block is inside frame
                    % Inline mse
                    MSE = sum( sum(  ( double(cur_frame(i:i+blockSize-1, j:j+blockSize-1)) - double(ref_frame(ref_y:ref_y+blockSize-1, ref_x:ref_x+blockSize-1)) ).^2 ./blockSize ) ./blockSize );
                    if MSE < MSE_min
                        MSE_min = MSE;
                        best_x = ldsp_ind(k,1);
                        best_y = ldsp_ind(k,2);
                    end
                end
            end
            dx = dx + best_x;
            dy = dy + best_y;
            if ( best_x == 0 && best_y ==0 )
                break;
            end
        end
        %sdsp
        sdsUpdate = 0;
        for k = 1:length(sdsp_ind)
            ref_y = i + dy + sdsp_ind(k,2);
            ref_x = j + dx + sdsp_ind(k,1);
            if ( ref_y > 0 && (ref_y + blockSize - 1) <= dim(1) ...
                    && ref_x > 0 && (ref_x + blockSize - 1) <= dim(2) ) %Check if block is inside frame
                % Inline mse
                MSE = sum( sum(  ( double(cur_frame(i:i+blockSize-1, j:j+blockSize-1)) - double(ref_frame(ref_y:ref_y+blockSize-1, ref_x:ref_x+blockSize-1)) ).^2 ./blockSize ) ./blockSize );
                if MSE < MSE_min
                    MSE_min = MSE;
                    best_x = sdsp_ind(k,1);
                    best_y = sdsp_ind(k,2);
                    sdsUpdate = 1;
                end
            end
        end
        if (sdsUpdate)
            dx = dx + best_x;
            dy = dy + best_y;
        end
        mvy( (i-1)/blockSize + 1, (j-1)/blockSize + 1 ) = dy;
        mvx( (i-1)/blockSize + 1, (j-1)/blockSize + 1 ) = dx;
    end
end

end



function [mvx, mvy] = fullSearch(cur_frame, ref_frame, N, R)
% Full search

dim = size(cur_frame);
mvx = zeros(dim / N);
mvy = zeros(dim / N);

for i = 1:N:dim(1)
    for j = 1:N:dim(2)
        MSE_min = 256^2;
        dy = 0;
        dx = 0;
        for k = -(R-1):1:(R-1)
            for l = -(R-1):1:(R-1) 
                if( i+k > 0 && i+k+N-1 <= dim(1) && j+l > 0 && j+l+N-1 <= dim(2) )
                    % Inline mse
                    MSE = sum( sum(  ( double(cur_frame(i:i+N-1, j:j+N-1)) - double(ref_frame(i+k:i+k+N-1, j+l:j+l+N-1)) ).^2 ./N ) ./N );
                    if MSE < MSE_min
                        MSE_min = MSE;
                        dy = k;
                        dx = l;
                    end
                end
            end
        end
        mvy( (i-1)/N + 1, (j-1)/N + 1 ) = dy;
        mvx( (i-1)/N + 1, (j-1)/N + 1 ) = dx;
    end
end

end