function [luma_subbands, Cb_subbands, Cr_subbands] = jp2k_gop_decoder(GOPnumber, GOPsize, inputFolder, outputFolder, frameWidth, frameHeight)

filePrefix = ['gop_' int2str(GOPnumber) '_sub_'];
filePostfix = 'j2c';

% Read lowpass subbands
y_temp = readJp2k(inputFolder, [filePrefix '1_luma.' filePostfix], outputFolder, frameWidth, frameHeight, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_LP_LUMA);
u_temp = readJp2k(inputFolder, [filePrefix '1_cb.' filePostfix], outputFolder, frameWidth/2, frameHeight/2, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_LP_CHROMA);
v_temp = readJp2k(inputFolder, [filePrefix '1_cr.' filePostfix], outputFolder, frameWidth/2, frameHeight/2, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_LP_CHROMA);

[m,n] = size(y_temp);
luma_subbands = zeros(m, n, GOPsize);
luma_subbands(:,:,1) = y_temp;

[m,n] = size(u_temp);
Cb_subbands = zeros(m, n, GOPsize);
Cb_subbands(:,:,1) = u_temp;

[m,n] = size(v_temp);
Cr_subbands = zeros(m, n, GOPsize);
Cr_subbands(:,:,1) = v_temp;

% Read highpass subbands (8bit)
for i = 2:GOPsize
    %luma_subbands(:,:,i) = ( double( readJp2k(inputFolder, [filePrefix int2str(i) '_luma.' filePostfix], outputFolder, frameWidth, frameHeight, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_HP_LUMA) ) - 127 )*2;
    [luma_temp, sf] = readJp2k(inputFolder, [filePrefix int2str(i) '_luma.' filePostfix], outputFolder, frameWidth, frameHeight, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_HP_LUMA, 1);
    luma_subbands(:,:,i) = ( double(luma_temp) - 127 ) * (1/sf);
    %Cb_subbands(:,:,i) = double( readJp2k(inputFolder, [filePrefix int2str(i) '_cb.' filePostfix], outputFolder, frameWidth/2, frameHeight/2, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_HP_CHROMA) ) - 127;
    [Cb_temp, sf2] = readJp2k(inputFolder, [filePrefix int2str(i) '_cb.' filePostfix], outputFolder, frameWidth/2, frameHeight/2, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_HP_CHROMA, 1);
    Cb_subbands(:,:,i) = ( double(Cb_temp) - 127 ) * (1/sf2);
    %Cr_subbands(:,:,i) = double( readJp2k(inputFolder, [filePrefix int2str(i) '_cr.' filePostfix], outputFolder, frameWidth/2, frameHeight/2, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_HP_CHROMA) ) - 127;
    [Cr_temp, sf3] = readJp2k(inputFolder, [filePrefix int2str(i) '_cr.' filePostfix], outputFolder, frameWidth/2, frameHeight/2, ConstantProperties.NO_JP2K_SPATIAL_LEVELS_HP_CHROMA, 1);
    Cr_subbands(:,:,i) = ( double(Cr_temp) - 127 ) * (1/sf3);
end

end


function [y, sf] = readJp2k(inputFolder, fileName, outputFolder, width, height, noSpatialLevels, varargin)
if ( ~exist([outputFolder '/reconstruced_subbands'], 'file') )
    mkdir([outputFolder '/reconstruced_subbands']);
end

fileIDin = fopen([inputFolder '/' fileName '.bin']);
fileIDout = fopen([outputFolder '/reconstruced_subbands/' fileName], 'w');

% Write missing header data to out file
fwrite(fileIDout, 65359, 'uint16', 0, 'b'); % Write SOC marker: oxFF4F
fwrite(fileIDout, 65361, 'uint16', 0, 'b'); % Write SIZ marker: oxFF51
fwrite(fileIDout, 41, 'uint16', 0, 'b'); % Write Lsiz: 41 bytes for 1 comp image
fwrite(fileIDout, 0, 'uint16', 0, 'b'); % Write CA: 0 for Part 1
fwrite(fileIDout, width, 'uint32', 0, 'b'); % Write F2
fwrite(fileIDout, height, 'uint32', 0, 'b'); % Write F1
fwrite(fileIDout, 0, 'uint32', 0, 'b'); % Write E2
fwrite(fileIDout, 0, 'uint32', 0, 'b'); % Write E1
fwrite(fileIDout, width, 'uint32', 0, 'b'); % Write T2
fwrite(fileIDout, height, 'uint32', 0, 'b'); % Write T1
fwrite(fileIDout, 0, 'uint32', 0, 'b'); % Write Omega2
fwrite(fileIDout, 0, 'uint32', 0, 'b'); % Write Omega1
fwrite(fileIDout, 1, 'uint16', 0, 'b'); % Write C
fwrite(fileIDout, 7, 'uint8', 0, 'b'); % Write B0
fwrite(fileIDout, 1, 'uint8', 0, 'b'); % Write S0_2
fwrite(fileIDout, 1, 'uint8', 0, 'b'); % Write S0_1
fwrite(fileIDout, 65362, 'uint16', 0, 'b'); % Write COD marker 0xFF52
fwrite(fileIDout, 12, 'uint16', 0, 'b'); % Write Lcod: 12 bytes
fwrite(fileIDout, 0, 'uint8', 0, 'b'); % Write CS
fwrite(fileIDout, 0, 'uint8', 0, 'b'); % Write O (progression order, LRCP)
fwrite(fileIDout, 1, 'uint16', 0, 'b'); % Write Lambda (no quality layers)
fwrite(fileIDout, 0, 'uint8', 0, 'b'); % Write MC (multicomponent transform)
fwrite(fileIDout, (noSpatialLevels-1), 'uint8', 0, 'b'); % Write D (no transform levels)
fwrite(fileIDout, 4, 'uint8', 0, 'b'); % Write E2 (block width = 4*2^E2)
fwrite(fileIDout, 4, 'uint8', 0, 'b'); % Write E1 (block height = 4*2^E2)
fwrite(fileIDout, 0, 'uint8', 0, 'b'); % Write MS (mode switches)
fwrite(fileIDout, 0, 'uint8', 0, 'b'); % Write WT (transform type, 0 = 9/7)
fwrite(fileIDout, 65372, 'uint16', 0, 'b'); % Write QCD marker: oxFF5C

if (nargin == 7)
    scalingFactor = fread(fileIDin, 1, 'ubit3');
    switch scalingFactor
        case 0
            sf = 0.5;
        case 1
            sf = 0.6;
        case 2
            sf = 0.7;
        case 3
            sf = 0.8;
        case 4
            sf = 0.9;
        case 5
            sf = 1;
        otherwise
            error('Bad scaling factor');
    end
else
    sf = 1;
end

%Write QCD segment (main header)
qcdSize = fread(fileIDin, 1, 'uint16', 0, 'b');
fwrite(fileIDout, qcdSize, 'uint16', 0, 'b');
fwrite(fileIDout, fread(fileIDin, qcdSize-2, 'uint8', 0, 'b'), 'uint8', 0, 'b');

%Write Tile header (SOT, Lsot, i_T, L_TP, i_TP, N_TP,t and SOD)
fwrite(fileIDout, 65424, 'uint16', 0, 'b'); % SOT = 0xFF90
fwrite(fileIDout, 10, 'uint16', 0, 'b'); % Lsot = 10 (0xa)
fwrite(fileIDout, 0, 'uint16', 0, 'b'); % i_T = 0
fwrite(fileIDout, 0, 'uint32', 0, 'b'); % Set L_TP to 0 (Acceptable when tile-part contains data up to (not including) EOC marker
fwrite(fileIDout, 0, 'uint8', 0, 'b'); % i_TP (tile-part index) = 0
fwrite(fileIDout, 1, 'uint8', 0, 'b'); % N_TP,t = 1 (no. tile-parts for this tile)
fwrite(fileIDout, 65427, 'uint16', 0, 'b'); % SOD marker (Start Of Data) = 0xFF93

%Write rest of file
fwrite(fileIDout, fread(fileIDin));

fclose(fileIDin);
fclose(fileIDout);

y = imread([outputFolder '/reconstruced_subbands/' fileName], 'j2c');
end