function reconstructed_frames = mctf_gop_decoder(subband_frames, mvx, mvy, numOfTemporalLevels, filterType, blockSize, varargin)
%GOP decoder

OBMC = 1;
if (nargin == 8)
    OBMC = varargin{2};
end

% Check parameters
if( ndims(subband_frames) ~= 3 )
    error('Frames parameter must contain 2 or more frames');
end

[frame_height, frame_width, GOP_size] = size(subband_frames);

if(GOP_size ~= 2^numOfTemporalLevels)
    error('The number of frames to decode must equal 2^numOfTemporalLevels');
end
if ( rem(blockSize, 1) ~= 0 )
    error('blockSize parameter must be integer valued');
end
if ( mod(frame_height, blockSize) ~= 0 || mod(frame_width, blockSize) ~= 0)
    error('Frame dimensions not divisible by Block size');
end
if ( size(mvx) ~= size(mvy) )
    error('mvx and mvy are not of equal size');
end

if( strcmpi(filterType, 'haar') )
    lowpass_frames = subband_frames(:, :, 1);
    level_index = 0;
    while( level_index < numOfTemporalLevels)
        lowpass_frames_temp = zeros(frame_height, frame_width, 2^(level_index+1));
        l_temp_counter = 1;
        for i = 1:2^level_index
            l_frame = lowpass_frames(:, :, i);
            h_frame = subband_frames(:, :, 2^level_index + i);
            [l_temp1, l_temp2] = HaarMCTF.synthesis(l_frame, h_frame, mvx(:, :, 2^level_index - 1 + i), mvy(:, :, 2^level_index - 1 + i), blockSize);
            
            lowpass_frames_temp(:, :, l_temp_counter) = l_temp1;
            lowpass_frames_temp(:, :, l_temp_counter+1) = l_temp2;
            l_temp_counter = l_temp_counter + 2;
            
        end
        level_index = level_index + 1;
        lowpass_frames = lowpass_frames_temp;
    end
    reconstructed_frames = uint8(lowpass_frames); %Convert from double to uint8
    
    
    
    
elseif ( strcmpi(filterType, '1/3') )
    
    lowpass_frames = zeros(frame_height, frame_width, 2);
    lowpass_frames(:,:,1) = varargin{1}; % IDR
    lowpass_frames(:,:,2) = subband_frames(:, :, 1);
    
    level_index = 0;
    mv_counter = 1;
    while( level_index < numOfTemporalLevels)
        lowpass_frames_temp = zeros(frame_height, frame_width, 2^(level_index+1)+1);
        l_temp_counter = 1;
        
        for i = 1:2^level_index
            l_frame_1 = lowpass_frames(:, :, i);
            l_frame_2 = lowpass_frames(:, :, i+1);
            h_frame = subband_frames(:, :, 2^level_index + i);
            
            l_temp = OneThreeMCTF.synthesis(l_frame_1, h_frame, l_frame_2, mvx(:, :, mv_counter), mvy(:, :, mv_counter), mvx(:, :, mv_counter+1), mvy(:, :, mv_counter+1), blockSize, OBMC);
            mv_counter = mv_counter + 2;
            
            lowpass_frames_temp(:, :, l_temp_counter) = l_frame_1;
            lowpass_frames_temp(:, :, l_temp_counter+1) = l_temp;
            lowpass_frames_temp(:, :, l_temp_counter+2) = l_frame_2;
            l_temp_counter = l_temp_counter + 2;
            
        end
        level_index = level_index + 1;
        lowpass_frames = lowpass_frames_temp;
    end
    reconstructed_frames = uint8(lowpass_frames(:,:,2:GOP_size+1)); %Convert from double to uint8
end


end