function [luma_mvx_rec, luma_mvy_rec, Cb_mvx_rec, Cb_mvy_rec, Cr_mvx_rec, Cr_mvy_rec] = motion_vector_gop_decoder(GOPnumber, noFieldsPerGOP, frameWidth, frameHeight, luma_blockSize, inputFolder, mvEntropyCoder, numberOfSymbols)

% mv_width = frameWidth / luma_blockSize; %Assumes no. blocks are equal in luma/chroma
% mv_height = frameHeight / luma_blockSize;

mv_width = (frameWidth / (luma_blockSize/2)-1); %TEST obmc
mv_height = (frameHeight / (luma_blockSize/2)-1);


% ==== Read motion data from stream =======================================
fileID = fopen([inputFolder '/gop_' num2str(GOPnumber) '_mv.bin']);

if (mvEntropyCoder == 1) 
    % Arithmetic coding
%     byteSequence = [];
%     while (~feof(fileID))
%         byteSequence = [byteSequence; fread(fileID, 1, 'uint8')];
%     end    
    byteSequence = fread(fileID);
%     mv_vector = Arith07(byteSequence);
%     mv_vector = mv_vector{1} - ( (numberOfSymbols+1)/2 - 1 );
    
    % Arithmetic decoding, then run-length decoding
    temp = Arith07(byteSequence);
    temp{1} = temp{1} - ( (numberOfSymbols+1)/2 - 1 );
    rleVals = cell(1,2);
    rleVals{1} = temp{1}';
    rleVals{2} = temp{2}';
    mv_vector = rle(rleVals);
else
    % Canonical Huffman
    
    noCoeffs = noFieldsPerGOP * mv_width * mv_height * 6;
    
    % Read number of symbols (1st byte in stream)
    noSymbolsLog2 = ceil( log2(numberOfSymbols) );
    
    %Read code lengths
    codeLengths = zeros(numberOfSymbols, 1);
    for i = 1:numberOfSymbols
        codeLengths(i) = fread(fileID, 1, ['ubit' num2str(noSymbolsLog2)]); % Read length of code word
    end
    
    % Construct code book
    canonicalCodebook = CanonicalHuffmanCodeBook( -(numberOfSymbols-1)/2 : (numberOfSymbols-1)/2, 'lengths', codeLengths );
    
    % Read coeffs
    mv_vector = zeros(noCoeffs,1);
    cntr = 1;
    readBitstring = [];
    while( ~feof(fileID) )
        bit = fread(fileID, 1, 'ubit1');    % read 1 bit
        readBitstring = [readBitstring bit];
        symbol = findSymbol(canonicalCodebook, readBitstring);
        if (symbol < Inf)
            mv_vector(cntr) = symbol;
            cntr = cntr + 1;
            readBitstring = [];
        end
    end
    
end

%Close file
fclose(fileID);

% Shape output
mvLumaW = mv_width;
mvLumaH = mv_height;
mvChromaW = frameWidth / luma_blockSize; %Assumes no. blocks are equal in luma/chroma, and Cb, Cr have equal no. vectors
mvChromaH = frameHeight / luma_blockSize;

luma_mvx_rec = zeros(mvLumaH, mvLumaW, noFieldsPerGOP);
luma_mvy_rec = zeros(mvLumaH, mvLumaW, noFieldsPerGOP);
Cb_mvx_rec = zeros(mvChromaH, mvChromaW, noFieldsPerGOP);
Cb_mvy_rec = zeros(mvChromaH, mvChromaW, noFieldsPerGOP);
Cr_mvx_rec = zeros(mvChromaH, mvChromaW, noFieldsPerGOP);
Cr_mvy_rec = zeros(mvChromaH, mvChromaW, noFieldsPerGOP);

for i = 1 : noFieldsPerGOP
    luma_mvx_rec(:,:,i) = reshape( mv_vector(1:mvLumaW*mvLumaH), mvLumaW, mvLumaH )';
    mv_vector = mv_vector( mvLumaW*mvLumaH + 1 : numel(mv_vector) );
    
    luma_mvy_rec(:,:,i) = reshape( mv_vector(1:mvLumaW*mvLumaH), mvLumaW, mvLumaH )';
    mv_vector = mv_vector( mvLumaW*mvLumaH + 1 : numel(mv_vector) );
    
    Cb_mvx_rec(:,:,i) = reshape( mv_vector(1:mvChromaW*mvChromaH), mvChromaW, mvChromaH )';
    mv_vector = mv_vector( mvChromaW*mvChromaH + 1 : numel(mv_vector) );
    
    Cb_mvy_rec(:,:,i) = reshape( mv_vector(1:mvChromaW*mvChromaH), mvChromaW, mvChromaH )';
    mv_vector = mv_vector( mvChromaW*mvChromaH + 1 : numel(mv_vector) );
    
    Cr_mvx_rec(:,:,i) = reshape( mv_vector(1:mvChromaW*mvChromaH), mvChromaW, mvChromaH )';
    mv_vector = mv_vector( mvChromaW*mvChromaH + 1 : numel(mv_vector) );
    
    Cr_mvy_rec(:,:,i) = reshape( mv_vector(1:mvChromaW*mvChromaH), mvChromaW, mvChromaH )';
    mv_vector = mv_vector( mvChromaW*mvChromaH + 1 : numel(mv_vector) );
end


end