% GOP mv encoder
function totalBytesWritten = motion_vector_gop_encoder(luma_mvx, luma_mvy, Cb_mvx, Cb_mvy, Cr_mvx, Cr_mvy, searchRange, GOPnumber, outputFolder, mvEntropyCoder)

% Reshape input into vector
dim = size(luma_mvx);
noFields = 1;
if (numel(dim) == 3)
    noFields = dim(3);
end
mv = zeros( 1, (numel(luma_mvx) * 2) + (numel(Cb_mvx) * 2) + (numel(Cr_mvx) * 2) );
cntr = 1;
noLumaCoeffs = numel( luma_mvx(:,:,1) );
noCbCoeffs = numel( Cb_mvx(:,:,1) );
noCrCoeffs = numel( Cr_mvx(:,:,1) );
for i = 1:noFields % Assuming mv arrays are in reverse t.level order (decoding order) (should be true, see mctf_gop_encoder)
    % Luma
    mv(cntr : cntr+noLumaCoeffs-1) = reshape( luma_mvx(:,:,i)', 1, noLumaCoeffs );
    cntr = cntr + noLumaCoeffs;

    mv(cntr : cntr+noLumaCoeffs-1) = reshape( luma_mvy(:,:,i)', 1, noLumaCoeffs );
    cntr = cntr + noLumaCoeffs;
    
    % Cb
    mv(cntr : cntr+noCbCoeffs-1) = reshape( Cb_mvx(:,:,i)', 1, noCbCoeffs );
    cntr = cntr + noCbCoeffs;

    mv(cntr : cntr+noCbCoeffs-1) = reshape( Cb_mvy(:,:,i)', 1, noCbCoeffs );
    cntr = cntr + noCbCoeffs;
    
    % Cr
    mv(cntr : cntr+noCrCoeffs-1) = reshape( Cr_mvx(:,:,i)', 1, noCrCoeffs );
    cntr = cntr + noCrCoeffs;

    mv(cntr : cntr+noCrCoeffs-1) = reshape( Cr_mvy(:,:,i)', 1, noCrCoeffs );
    cntr = cntr + noCrCoeffs;
end

fileID = fopen([outputFolder '/gop_' num2str(GOPnumber) '_mv.bin'], 'w');

if (mvEntropyCoder == 1)
    % Arithmetic coding
    
    % Run-length encoding
    mvCell = rle(mv);
    mvCell{1} = mvCell{1} + (searchRange-1);
    % Arithmetic coding    
    y = Arith07(mvCell);
    totalBytesWritten = fwrite(fileID, y, 'uint8');
    
else
    % Canonical Huffman
    
    symbols = -(searchRange-1) : (searchRange-1);
    frequencies = histc( mv, symbols );    %get freqs
    
    
    % Get codebook
    codeBook = CanonicalHuffmanCodeBook(symbols, 'freqs', frequencies);
    
    % *** Write compressed mv data to file ***
    totalBitsWritten = 0;
        
    noSymbolsLog2 = ceil( log2(length(symbols)) );
    
    % Write codeword length for all symbols 8 (bit)
    for i = 1:length(codeBook.codeLengths)        
        totalBitsWritten = totalBitsWritten + 8 * fwrite(fileID, codeBook.codeLengths(i,2), ['ubit' num2str(noSymbolsLog2)]);
    end
    
    % Write coded coefficients
    for i = 1:numel(mv)
        codeword = findCodeword(codeBook, mv(i));
        totalBitsWritten = totalBitsWritten + fwrite(fileID, codeword, 'ubit1');
    end
    % (totalBitsWritten is now equal to total number of bits written to file)
    totalBytesWritten = ceil(totalBitsWritten / 8);
    
end

% Close file
fclose(fileID);

end
